﻿#region license

/*
EasyBuildSQL - is a C# library oriented to help to developers to build sql sentence in differents database engine. 
EasyBuildSQL is developed to .NET Framework.
 
Copyright (C) 2010 - Alejandro Genovesi
http://sourceforge.net/projects/easybuildsql/ 
Contributor : Arturo Martinez

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 
*/

#endregion




using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using System.Data.Common;
using System.Data.SqlClient;

namespace EasyBuildSQL
{
    public class ebConnection
    {
        //Active DataBase
        protected DataBaseType FDataBase = DataBaseType.MSSQL;
        protected string FUserName, FServer, FDBName, FPassword, FConnectionString, FConnectionError;

        //GenericDbConnection
        protected DbConnection FConnection;

        public DataBaseType DataBase
        {
            get { return FDataBase; }
        }

        public string UserName
        {
            get { return FUserName; }
            set { FUserName = value; }
        }

        public string Server
        {
            get { return FServer; }
            set { FServer = value; }
        }

        public string DBName
        {
            get { return FDBName; }
            set { FDBName = value; }
        }

        public string Password
        {
            get { return FPassword; }
            set { FPassword = value; }
        }

        public string ConnectionString
        {
            get { return ReturnConnectionString(); }
        }

        public DbConnection Connection
        {
            get { return FConnection; }
        }

        public string ConnectionError
        {
            get { return FConnectionError; }
        }

        protected void PrepareebConnection(DataBaseType DB, string UserName, string Password, string DBName, string Server)
        {

            FDataBase = DB;
            FUserName = UserName;
            FServer = Server;
            FDBName = DBName;
            FPassword = Password;

            ReturnConnectionString();

            switch (FDataBase)
            {

                case DataBaseType.MSSQL:
                    {
                        FConnection = new SqlConnection(FConnectionString);
                    }
                    break;

                case DataBaseType.MySQL:
                    {
                        FConnection = new MySqlConnection(FConnectionString);
                    }
                    break;

            }

        }

        protected string ReturnConnectionString()
        {
            switch (FDataBase)
            {
                case DataBaseType.MSSQL:
                    {
                        FConnectionString = string.Format("Data Source={0};Initial Catalog={1};Persist Security Info=True;User ID={2};Password={3};Trusted_Connection=False", FServer, FDBName, FUserName, FPassword);

                    }
                    break;

                case DataBaseType.MySQL:
                    {
                        FConnectionString = string.Format("Data Source={0};Database={1};User Id={2};Password={3}; Allow User Variables=True", FServer, FDBName, FUserName, FPassword);
                    }
                    break;

            }
            return FConnectionString;
        }

        /// <summary>
        /// Class constructor 
        /// </summary>
        /// <param name="DB">DataBaseType (MSSQL or MySQL)</param>
        /// <param name="UserName">DB Username</param>
        /// <param name="Password">DB Password</param>
        /// <param name="DBName">DataBase Name</param>
        /// <param name="Server">DataBase Server</param>
        public ebConnection(DataBaseType DB, string UserName, string Password, string DBName, string Server)
        {
            PrepareebConnection(DB, UserName, Password, DBName, Server);
        }
        /// <summary>
        /// Class constructor alternative
        /// </summary>
        /// <param name="DB">DataBaseType (MSSQL or MySQL)</param>
        /// <param name="Credentials">Database connectios credentiales (UserName, Password, DBName and ServerName)</param>
        /// <summary>
        public ebConnection(ConnectionCredentials Credentials)
        {

            PrepareebConnection(Credentials.ObtainDBType(), Credentials["UserName"], Credentials["Password"], Credentials["DBName"], Credentials["Server"]);
        }

        public Boolean Open()
        {
            try
            {
                FConnection.Open();
                FConnectionError = "";
                return true;

            }
            catch (Exception e)
            {
                FConnectionError = e.Message;
                return false;

            }

        }

        public void Close()
        {
            try
            {
                FConnection.Close();
                FConnectionError = "";

            }
            catch (Exception e)
            {
                FConnectionError = e.Message;
            }

        }


    }

    /// <summary>
    /// Clase que se encarga de guardar valores de conexion
    /// </summary>
    public class ConnectionCredentials : Dictionary<string, string>
    {

        /// <summary>
        /// Class constructor 
        /// </summary>
        /// <param name="UserName">DB Username</param>
        /// <param name="Password">DB Password</param>
        /// <param name="DBName">DataBase Name</param>
        /// <param name="Server">DataBase Server</param>
        public ConnectionCredentials(string UserName, string Password, string DBName, string Server, string DBType)
        {
            Clear();
            Add("UserName", UserName);
            Add("Password", Password);
            Add("DBName", DBName);
            Add("Server", Server);
            Add("DBType", DBType);
        }

        /// <summary>
        /// Function to return DataBaseType
        /// </summary>
        /// <returns></returns>
        public DataBaseType ObtainDBType()
        {
            string sDB = string.Empty;
            DataBaseType DB = DataBaseType.MSSQL;

            try
            {
                sDB = this["DBType"];
            }
            catch (KeyNotFoundException)
            {
                sDB = "MSSQL";
            }

            switch (sDB)
            {
                case "MSSQL":
                    return DataBaseType.MSSQL;
                    break;

                case "MySQL":
                    return DataBaseType.MySQL;
                    break;

                default:
                    return DataBaseType.MSSQL;
                    break;
            }
        }

        /// <summary>
        /// Function to assign a DataBaseType KeyValuePair
        /// </summary>
        /// <returns></returns>
        public void AssignDBType(DataBaseType DBType)
        {
            string sDB = "MSSQL";

            switch (DBType)
            {
                case DataBaseType.MSSQL:
                    sDB = "MSSQL";
                    break;

                case DataBaseType.MySQL:
                    sDB = "MySQL";
                    break;

                default:
                    sDB = "MSSQL";
                    break;
            }

            try
            {
                this["DBType"] = sDB;
            }
            catch (KeyNotFoundException)
            {
                Add("DBType", "MSSQL");
            }
        
        }


    }

}
