#region license

/*
EasyBuildSQL - is a C# library oriented to help to developers to build sql sentence in differents database engine. 
EasyBuildSQL is developed to .NET Framework.
 
Copyright (C) 2010 - Alejandro Genovesi
http://sourceforge.net/projects/easybuildsql/ 
Contributor : Arturo Martinez

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 
*/

#endregion
using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Collections;
using System.Data.Common;
using System.IO;


namespace EasyBuildSQL
{
  public abstract class ebDataSet : ebSQL
  {
      public ebDataSet(string UserName, string Password, string DBName, string Server, DataBaseType DBType)
      : base(UserName, Password, DBName, Server, DBType)
    {
    }

      public ebDataSet(ConnectionCredentials Credentials)
      : base(Credentials)
    {
    }

      

    protected bool fInited;

    protected override void Initialize()
    {
        base.Initialize();
      
      fInited = true;
    }

    private string ParameterName(string Name)
	{
        if (this is ebQuery)
        {
                    if (Name[0] != '?')
                        Name = '?' + Name;

                    return Name.ToUpper();

        }
        else
        {

            switch (fDBType)
            {
                case DataBaseType.MSSQL:
                    if (Name[0] != '@')
                        Name = '@' + Name;

                    return Name.ToUpper();

                case DataBaseType.MySQL:
                    if (Name[0] == '@')
                        Name = Name.Replace("@", "");

                    return "v_" + Name.ToUpper();

                default:
                    if (Name[0] == '@')
                        Name = Name.Replace("@", "");

                    return Name.ToUpper();
            }
        }
    }


    protected DbParameter CreateParameter(string Name, DbType Type, ParameterDirection Direction)
    {
      return CreateParameter(Name, Type, Direction, null, 0);
    }

    protected DbParameter CreateParameter(string Name, DbType Type, ParameterDirection Direction, object Value)
    {
      return CreateParameter(Name, Type, Direction, Value, 0);
    }

    protected DbParameter CreateParameter(string Name, DbType Type, ParameterDirection Direction, object Value, int Large)
    {
      if (!fInited)
          Initialize();

      DbParameter pNew = fCommand.CreateParameter();

      pNew.DbType = Type;
      pNew.Direction = Direction;
      pNew.ParameterName = ParameterName(Name);

      if (Value != null)
          pNew.Value = Value;

      if (Large != 0)
          pNew.Size = Large;

      fCommand.Parameters.Add(pNew);

      return pNew;
    }

    public DbParameter Parameter(string Name)
    {
      return fCommand.Parameters[ParameterName(Name)];
    }

    public bool AddParameterBLOB(string Parameter, string FileName)
    {
      bool result = false;

      DbParameter pBlob = CreateParameter(Parameter, DbType.Binary, ParameterDirection.Input);

      if (string.IsNullOrEmpty(FileName))
        pBlob.Value = DBNull.Value;
      else
      {
        byte[] buffer = BlobSQL.LoadFromFile(FileName);

        if (buffer.Length > 0)
        {
          pBlob.Value = buffer;

          result = true;
        }
      }

      return result;
    }

    public bool AddParameterBLOB(string Parameter, Stream Stream)
    {
        bool result = false;

        DbParameter pBlob = CreateParameter(Parameter, DbType.Binary, ParameterDirection.Input);

        if (Stream.Length != 0)
        {
            byte[] buffer = BlobSQL.LoadFromStream(Stream);

            if (buffer.Length > 0)
            {
                pBlob.Value = buffer;
                result = true;
            }

        }
        return result;

    }

    public abstract DbDataReader OpenDataReader();
    public abstract DataSet OpenDataSet();
  }
}
