#region license

/*
EasyBuildSQL - is a C# library oriented to help to developers to build sql sentence in differents database engine. 
EasyBuildSQL is developed to .NET Framework.
 
Copyright (C) 2010 - Alejandro Genovesi
http://sourceforge.net/projects/easybuildsql/ 
Contributor : Arturo Martinez

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 
*/

#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace EasyBuildSQL
{
    public static class ebField
    {
        public static string AsString(object Field)
        {
            if (Convert.IsDBNull(Field))
                return "";
            else
                return Field.ToString();
        }

        public static int AsInteger(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToInt32(Field);
        }

        public static Int64 AsInt64(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToInt64(Field);
        }

        public static bool AsBoolean(object Field)
        {
            if (Convert.IsDBNull(Field))
                return false;
            else
                return Convert.ToBoolean(Field);
        }

        public static DateTime AsDateTime(object Field)
        {
            if (Convert.IsDBNull(Field))
                return DateTime.MinValue;
            else
                return Convert.ToDateTime(Field);
        }

        public static double AsDouble(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToDouble(Field);
        }

        public static Single AsSingle(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToSingle(Field);
        }

        public static byte AsByte(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToByte(Field);
        }

        public static char AsChar(object Field)
        {
            if (Convert.IsDBNull(Field))
                return '\0';
            else
                return Convert.ToChar(Field);
        }

        public static UInt16 AsWord(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToUInt16(Field);
        }

        public static UInt32 AsLong(object Field)
        {
            if (Convert.IsDBNull(Field))
                return 0;
            else
                return Convert.ToUInt32(Field);
        }
    }

    public static class ebBLOBField
    {
        public static bool SaveToFile(object Field, string Directory, string FileName)
        {
            bool Result = false;
            try
            {
                //If Field is null
                if (Field == null)
                {
                    throw new ArgumentNullException("Field is null");
                }
                //or field not is Byte[]
                else if (Field.GetType().FullName != "System.Byte[]")
                {
                    throw new ArgumentException("Field must be Byte[]");
                }
                //or field is empty
                else if (Convert.IsDBNull(Field))
                {
                    return false;
                }

                //The FullName of File
                string FileFullName = Path.Combine(Directory,FileName);

                if (File.Exists(FileFullName))
                    File.Delete(FileFullName);

                FileStream fsFile = new FileStream(FileFullName, FileMode.Create);
                BinaryWriter bwFile = new BinaryWriter(fsFile);

                Byte[] BufferField = (Byte[])Field;

                bwFile.Write(BufferField);
                bwFile.Flush();
                bwFile.Close();
                fsFile.Close();

                //Operation is ok
                Result = true;
            }
            catch
            {
                Result = false;
            }

            return Result;
        }


        public static string SaveToFile( string Directory, string FileExtension, object Field)
        {

            //If Directory is empty then assign the temporary folder
            if (Directory == string.Empty)
                Directory = Path.GetTempPath();

            //Assign the RandomName to fileName
            string SimpleFileName = Path.GetRandomFileName();
            //If FileExtension <> "" then change the file extension
            if (FileExtension != string.Empty)
                SimpleFileName = Path.ChangeExtension(SimpleFileName, FileExtension);

            if (SaveToFile(Field, Directory, SimpleFileName))
                return Path.Combine(Directory, SimpleFileName);
            else
                return string.Empty;
        }

        public static string SaveToFile(object Field, string FileExtension)
        {

            //Assign the temporary folder
            string Directory = Path.GetTempPath();

            return SaveToFile( Directory, FileExtension, Field);
        }

        public static MemoryStream SaveToMemoryStream(object Field)
        {
            MemoryStream Result = null;
            try
            {
                //If Field is null
                if (Field == null)
                {
                    throw new ArgumentNullException("Field is null");
                }
                //or field not is Byte[]
                else if (Field.GetType().FullName != "System.Byte[]")
                {
                    throw new ArgumentException("Field must be Byte[]");
                }
                //or field is empty
                else if (Convert.IsDBNull(Field))
                {
                    return null;
                }

                Result = new MemoryStream((Byte[])Field);
            }
            catch
            {
                Result = null;
            }

            return Result;
        }

    
    }
}
