#region license

/*
EasyBuildSQL - is a C# library oriented to help to developers to build sql sentence in differents database engine. 
EasyBuildSQL is developed to .NET Framework.
 
Copyright (C) 2010 - Alejandro Genovesi
http://sourceforge.net/projects/easybuildsql/ 
Contributor : Arturo Martinez

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.

 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 
*/

#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.ComponentModel;
using System.Data.Common;

namespace EasyBuildSQL
{
  public class ebQuery : ebDataSet
  {
    private StringBuilder fSQL = null;
    private bool fUseTransaction;

    public ebQuery(string UserName, string Password, string DBName, string Server, DataBaseType DBType)
        : base(UserName, Password, DBName, Server, DBType)
    {
    }

    public ebQuery(ConnectionCredentials Credentials)
        : base(Credentials)
    {
    }
      
    private void Finish()
    {
      if (fSQL == null)
        fCommand.CommandText = string.Empty;
      else
      {
        if (fUseTransaction)
        {
          switch (fDBType)
          {
            case DataBaseType.MSSQL:
              fSQL.Append(MSSQL.COMMIT);
              break;

            case DataBaseType.MySQL:
              fSQL.Append(MySQL.COMMIT);
              break;

          }
        }

        if (fDBType == DataBaseType.MSSQL)
        {
            //I must change the question simbol by @ in parameters
            foreach (DbParameter Par in fCommand.Parameters)
            {
                string OldName = Par.ParameterName;
                string NewName = Par.ParameterName.Replace("?", "@");
                fSQL = fSQL.Replace(OldName, NewName);
                Par.ParameterName = NewName;

            }
            
            // se asigna el cdigo SQL al comando que la ejecutar en el servidor
          fCommand.CommandText = fSQL.ToString().Trim();
        }
        else
        {

          fCommand.CommandText = fSQL.ToString().Trim();

          if (!fCommand.CommandText.EndsWith(";"))
            fCommand.CommandText += ";";
        }
      }
    }

    protected override void Initialize()
    {
        base.Initialize();

      fSQL = new StringBuilder();

      if (fUseTransaction)
        switch (fDBType)
        {
          case DataBaseType.MSSQL:
            fSQL.AppendLine(MSSQL.BEGIN_TRANSACTION);
            break;

          case DataBaseType.MySQL:
            fSQL.AppendLine(MySQL.BEGIN_TRANSACTION);
            break;

        }
      else if (fDBType == DataBaseType.MSSQL)
          fSQL.AppendLine(MSSQL.CONNECTION_OPTIONS);
    }

    public void Init(bool Clean, bool UseTransaction)
    {
      if ((fSQL == null) || Clean || UseTransaction)
      {
        fUseTransaction = UseTransaction;

        Initialize();
      }
    }

    public void Init()
    {
      Init(true, false);
    }

    public void Init(bool Clean)
    {
      Init(Clean, false);
    }

    public string SQL
    {
      get
      {
        if (fSQL == null)
          return "";
        else
          return fSQL.ToString();
      }
    }

    public DbParameter AddParameter(string Name, DbType Type, object Value, int Large)
    {
      return CreateParameter(Name, Type, ParameterDirection.Input, Value, Large);
    }

    public DbParameter AddParameter(string Name, DbType Type, object Value)
    {
      return CreateParameter(Name, Type, ParameterDirection.Input, Value, 0);
    }

    public void AddSQL(string SQL)
    {
      if (!fInited)
        Initialize();

      fSQL.AppendLine(SQL);
    }

    public override bool Execute()
    {
      Finish();

      try
      {
        fConnection.Open();
       
        fCommand.Connection = fConnection.Connection;
        fCommand.ExecuteNonQuery();

        fConnection.Close();

        return true;
      }
      catch (Exception e)
      {
        fConnection.Close();
        fErrorSQL = e.Message;

        return false;
      }
    }

    public override DbDataReader OpenDataReader()
    {
      Finish();

      try
      {
        DbDataReader drResult;

        fConnection.Open();

        fCommand.Connection = fConnection.Connection;

        drResult = fCommand.ExecuteReader(CommandBehavior.CloseConnection);

        return drResult;
      }
      catch (Exception e)
      {
        fConnection.Close();
        fErrorSQL = e.Message;

        return null;
      }
    }

    public override DataSet OpenDataSet()
    {
      Finish();

      try
      {
        DataSet dsResult = new DataSet();
        DbDataAdapter Adapter;

        fConnection.Open();

        fCommand.Connection = fConnection.Connection;

        switch (fDBType)
        {
            case DataBaseType.MSSQL:
                {
                    Adapter = new SqlDataAdapter( (SqlCommand)fCommand );
                    Adapter.Fill(dsResult);
                    break;
                }

            case DataBaseType.MySQL:
                {
                    Adapter = new MySql.Data.MySqlClient.MySqlDataAdapter((MySql.Data.MySqlClient.MySqlCommand)fCommand);
                    Adapter.Fill(dsResult);
                    break;
                }
        }
        
        

        fConnection.Close();

        return dsResult;
      }
      catch (Exception e)
      {
        fConnection.Close();
        fErrorSQL = e.Message;

        return null;
      }
    }
  }
}
